# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Grr Shell utility methods and constants."""

import datetime
from typing import Union


WINDOWS = 'Windows'
DARWIN = 'Darwin'
LINUX = 'Linux'
OS_TYPES = (WINDOWS, DARWIN, LINUX)


def UnixTSToReadable(ts: Union[float, int]) -> str:
  """Converts a unix timestamp (seconds) to a ISO8601 string.

  Args:
    ts: A Unix timestamp

  Returns:
    A human readbable string for the timestamp.
  """
  return datetime.datetime.fromtimestamp(
      int(ts),
      tz=datetime.timezone.utc).strftime('%Y-%m-%dT%H:%M:%SZ')
